@@ -41,6 +41,7 @@ from . import __version__, utils
 
 log = logging.getLogger(__name__)
 
+
 async def json_or_text(response):
     text = await response.text(encoding='utf-8')
     try:
@@ -52,13 +53,14 @@ async def json_or_text(response):
 
     return text
 
-class Route:
-    BASE = 'https://discord.com/api/v7'
 
-    def __init__(self, method, path, base=BASE, **parameters):
+class Route:
+    BASE = 'https://discord.com/api/v9'
+
+    def __init__(self, method, path, **parameters):
         self.path = path
         self.method = method
-        url = (base + self.path)
+        url = (self.BASE + self.path)
         if parameters:
             self.url = url.format(**{k: _uriquote(v) if isinstance(v, str) else v for k, v in parameters.items()})
         else:
@@ -73,6 +75,7 @@ class Route:
         # the bucket is just method + path w/ major parameters
         return '{0.channel_id}:{0.guild_id}:{0.path}'.format(self)
 
+
 class MaybeUnlock:
     def __init__(self, lock):
         self.lock = lock
@@ -90,21 +93,21 @@ class MaybeUnlock:
 
 # For some reason, the Discord voice websocket expects this header to be
 # completely lowercase while aiohttp respects spec and does it as case-insensitive
+
+
 aiohttp.hdrs.WEBSOCKET = 'websocket'
 
+
 class HTTPClient:
     """Represents an HTTP client sending HTTP requests to the Discord API."""
 
     SUCCESS_LOG = '{method} {url} has received {text}'
     REQUEST_LOG = '{method} {url} with {json} has returned {status}'
 
-    # we have to overwride that beacuse interactions are using v8
-    V8BASE = 'https://discord.com/api/v8'
-
     def __init__(self, connector=None, *, proxy=None, proxy_auth=None, loop=None, unsync_clock=True):
         self.loop = asyncio.get_event_loop() if loop is None else loop
         self.connector = connector
-        self.__session = None # filled in static_login
+        self.__session = None  # filled in static_login
         self._locks = weakref.WeakValueDictionary()
         self._global_over = asyncio.Event()
         self._global_over.set()
@@ -354,7 +357,7 @@ class HTTPClient:
 
         return self.request(Route('POST', '/users/@me/channels'), json=payload)
 
-    def send_message(self, channel_id, content, *, tts=False, embed=None, components=None, nonce=None, allowed_mentions=None, message_reference=None):
+    def send_message(self, channel_id, content, *, tts=False, embeds=None, components=None, nonce=None, allowed_mentions=None, message_reference=None):
         r = Route('POST', '/channels/{channel_id}/messages', channel_id=channel_id)
         payload = {}
 
@@ -364,8 +367,8 @@ class HTTPClient:
         if tts:
             payload['tts'] = True
 
-        if embed:
-            payload['embed'] = embed
+        if embeds:
+            payload['embeds'] = embeds
 
         if components:
             payload['components'] = components
@@ -384,15 +387,15 @@ class HTTPClient:
     def send_typing(self, channel_id):
         return self.request(Route('POST', '/channels/{channel_id}/typing', channel_id=channel_id))
 
-    def send_files(self, channel_id, *, files, content=None, tts=False, embed=None, components=None, nonce=None, allowed_mentions=None, message_reference=None):
+    def send_files(self, channel_id, *, files, content=None, tts=False, embeds=None, components=None, nonce=None, allowed_mentions=None, message_reference=None):
         r = Route('POST', '/channels/{channel_id}/messages', channel_id=channel_id)
         form = []
 
         payload = {'tts': tts}
         if content:
             payload['content'] = content
-        if embed:
-            payload['embed'] = embed
+        if embeds:
+            payload['embeds'] = embeds
         if components:
             payload['components'] = components
         if nonce:
@@ -435,11 +438,10 @@ class HTTPClient:
         return self.request(r, reason=reason)
 
     def delete_messages(self, channel_id, message_ids, *, reason=None):
-        r = Route('POST', '/channels/{channel_id}/messages/bulk_delete', channel_id=channel_id)
+        r = Route('POST', '/channels/{channel_id}/messages/bulk-delete', channel_id=channel_id)
         payload = {
             'messages': message_ids
         }
-
         return self.request(r, json=payload, reason=reason)
 
     def edit_message(self, channel_id, message_id, **fields):
@@ -448,18 +450,90 @@ class HTTPClient:
 
     def post_initial_response(self, use_webhook, _resp, interaction_id, token, application_id):
         r_url = f"/webhooks/{application_id}/{token}/callback" if use_webhook is True else f"/interactions/{interaction_id}/{token}/callback"
-        r = Route("POST", r_url, self.V8BASE)
+        r = Route("POST", r_url)
         return self.request(r, json=_resp)
 
-    def edit_interaction_response(self, use_webhook, interaction_id, token, application_id, deffered, **fields):
-        if not deffered:
-            fields = {'data': fields}
-            fields['type'] = 7
-            r = Route("POST", f'/webhooks/{application_id}/{token}/callback' if use_webhook is True else f"/interactions/{interaction_id}/{token}/callback", self.V8BASE)
+    def edit_interaction_response(self, use_webhook, interaction_id, token, application_id, deferred, files=None, **fields):
+        if not deferred:
+            fields = {'data': fields, 'type': 7}
+            r = Route('POST', f'/webhooks/{application_id}/{token}/callback' if use_webhook is True else f"/interactions/{interaction_id}/{token}/callback")
         else:
-            r = Route('PATCH', f'/webhooks/{application_id}/{token}/messages/@original', self.V8BASE)
-        return self.request(r, json=fields)
+            r = Route('PATCH', f'/webhooks/{application_id}/{token}/messages/@original')
+        form = []
+        if files is not None:
+            form.append({'name': 'payload_json', 'value': utils.to_json(fields)})
+            if len(files) == 1:
+                file = files[0]
+                form.append({
+                    'name': 'file',
+                    'value': file.fp,
+                    'filename': file.filename,
+                    'content_type': 'application/octet-stream'
+                })
+            else:
+                for index, file in enumerate(files):
+                    form.append({
+                        'name': 'file%s' % index,
+                        'value': file.fp,
+                        'filename': file.filename,
+                        'content_type': 'application/octet-stream'
+                    })
 
+            return self.request(r, form=form, files=files)
+        else:
+            return self.request(r, json=fields)
+
+    def send_interaction_response(self, use_webhook, interaction_id, token, application_id, deferred, followup,
+                                  *, content=None, tts=False, embeds=None, components=None, files=None, nonce=None,
+                                  allowed_mentions=None, message_reference=None, flags=None):
+        form = []
+        payload = {'tts': tts}
+        if content:
+            payload['content'] = content
+        if embeds:
+            payload['embeds'] = embeds
+        if components:
+            payload['components'] = components
+        if nonce:
+            payload['nonce'] = nonce
+        if allowed_mentions:
+            payload['allowed_mentions'] = allowed_mentions
+        if message_reference:
+            payload['message_reference'] = message_reference
+        if flags:
+            payload['flags'] = flags
+        if not deferred and not followup:
+            payload = {'type': 4, 'data': payload}
+            r = Route('POST', f'/webhooks/{application_id}/{token}/callback' if use_webhook is True else f"/interactions/{interaction_id}/{token}/callback")
+        else:
+            r = Route('POST', f'/webhooks/{application_id}/{token}')
+        if files is not None:
+            form.append({'name': 'payload_json', 'value': utils.to_json(payload)})
+            if len(files) == 1:
+                file = files[0]
+                form.append({
+                    'name': 'file',
+                    'value': file.fp,
+                    'filename': file.filename,
+                    'content_type': 'application/octet-stream'
+                })
+            else:
+                for index, file in enumerate(files):
+                    form.append({
+                        'name': 'file%s' % index,
+                        'value': file.fp,
+                        'filename': file.filename,
+                        'content_type': 'application/octet-stream'
+                    })
+
+            return self.request(r, form=form, files=files)
+        else:
+            return self.request(r, json=payload)
+
+    def get_original_interaction_response(self, interaction_token, application_id):
+        r = Route('GET', f'/webhooks/{application_id}/{interaction_token}/messages/@original')
+        return self.request(r)
+        
     def add_reaction(self, channel_id, message_id, emoji):
         r = Route('PUT', '/channels/{channel_id}/messages/{message_id}/reactions/{emoji}/@me',
                   channel_id=channel_id, message_id=message_id, emoji=emoji)
@@ -492,7 +566,7 @@ class HTTPClient:
 
     def clear_single_reaction(self, channel_id, message_id, emoji):
         r = Route('DELETE', '/channels/{channel_id}/messages/{message_id}/reactions/{emoji}',
-                   channel_id=channel_id, message_id=message_id, emoji=emoji)
+                  channel_id=channel_id, message_id=message_id, emoji=emoji)
         return self.request(r)
 
     def get_message(self, channel_id, message_id):
@@ -990,7 +1064,7 @@ class HTTPClient:
     def application_info(self):
         return self.request(Route('GET', '/oauth2/applications/@me'))
 
-    async def get_gateway(self, *, encoding='json', v=6, zlib=True):
+    async def get_gateway(self, *, encoding='json', v=9, zlib=True):
         try:
             data = await self.request(Route('GET', '/gateway'))
         except HTTPException as exc:
@@ -1001,7 +1075,7 @@ class HTTPClient:
             value = '{0}?encoding={1}&v={2}'
         return value.format(data['url'], encoding, v)
 
-    async def get_bot_gateway(self, *, encoding='json', v=6, zlib=True):
+    async def get_bot_gateway(self, *, encoding='json', v=9, zlib=True):
         try:
             data = await self.request(Route('GET', '/gateway/bot'))
         except HTTPException as exc:
