@@ -33,16 +33,13 @@ import io
 
 from . import utils
 from .reaction import Reaction
-try:
-    from .emoji import Emoji
-except ImportError:
-    pass
+from .emoji import Emoji
 from .partial_emoji import PartialEmoji
 from .calls import CallMessage
 from .enums import MessageType, ChannelType, try_enum
-from .errors import InvalidArgument, ClientException, HTTPException, NotFound, UnknowInteraction
+from .errors import InvalidArgument, ClientException, HTTPException, NotFound
 from .embeds import Embed
-from .components import DecodeMessageComponents, Button, DropdownMenue, ActionRow
+from .components import Button, SelectMenu, ActionRow
 from .member import Member
 from .flags import MessageFlags
 from .file import File
@@ -75,7 +72,9 @@ def convert_emoji_reaction(emoji):
 
     raise InvalidArgument('emoji argument must be str, Emoji, or Reaction not {.__class__.__name__}.'.format(emoji))
 
+
 class Attachment(Hashable):
+
     """Represents an attachment from Discord.
 
     .. container:: operations
@@ -268,6 +267,7 @@ class Attachment(Hashable):
         data = await self.read(use_cached=use_cached)
         return File(io.BytesIO(data), filename=self.filename, spoiler=spoiler)
 
+
 class DeletedReferencedMessage:
     """A special sentinel type that denotes whether the
     resolved message referenced message had since been deleted.
@@ -300,6 +300,7 @@ class DeletedReferencedMessage:
 
 
 class MessageReference:
+
     """Represents a reference to a :class:`~discord.Message`.
 
     .. versionadded:: 1.5
@@ -454,7 +455,7 @@ class Message(Hashable):
         in cases where it might be a system message for :attr:`system_content`.
     author: :class:`abc.User`
         A :class:`Member` that sent the message. If :attr:`channel` is a
-        private channel or the user has the left the guild, then it is a :class:`User` instead.
+        private channel or the user has left the guild, then it is a :class:`User` instead.
     content: :class:`str`
         The actual contents of the message.
     nonce
@@ -552,12 +553,12 @@ class Message(Hashable):
 
     def __init__(self, *, state, channel, data):
         self._state = state
-        self.id = int(data['id'])
+        self.id = utils._get_as_snowflake(data, 'id')
         self.webhook_id = utils._get_as_snowflake(data, 'webhook_id')
         self.reactions = [Reaction(message=self, data=d) for d in data.get('reactions', [])]
-        self.attachments = [Attachment(data=a, state=self._state) for a in data['attachments']]
-        self.embeds = [Embed.from_dict(a) for a in data['embeds']]
-        self.components = data.get('components', [])
+        self.attachments = [Attachment(data=a, state=self._state) for a in data.get('attachments', [])]
+        self.embeds = [Embed.from_dict(a) for a in data.get('embeds', [])]
+        self.components = [ActionRow.from_dict(d) for d in data.get('components', [])]
         self.application = data.get('application')
         self.activity = data.get('activity')
         self.channel = channel
@@ -842,7 +843,7 @@ class Message(Hashable):
         .. note::
 
             This *does not* affect markdown. If you want to escape
-            or remove markdown then use :func:`utils.escape_markdown` or :func:`utils.remove_markdown` 
+            or remove markdown then use :func:`utils.escape_markdown` or :func:`utils.remove_markdown`
             respectively, along with this function.
         """
 
@@ -996,6 +997,28 @@ class Message(Hashable):
         if self.type is MessageType.guild_discovery_grace_period_final_warning:
             return 'This server has failed Discovery activity requirements for 3 weeks in a row. If this server fails for 1 more week, it will be removed from Discovery.'
 
+    @property
+    def all_components(self):
+        """Returns all :class:`Button`'s and :class:`SelectMenu`'s that are contained in the message"""
+        for action_row in self.components:
+            for component in action_row:
+                yield component
+
+    @property
+    def all_buttons(self):
+        """Returns all :class:`Button`'s that are contained in the message"""
+        for action_row in self.components:
+            for component in action_row:
+                yield component
+
+    @property
+    def all_select_menus(self):
+        """Returns all :class:`SelectMenu`'s that are contained in the message"""
+        for action_row in self.components:
+            for component in action_row:
+                if isinstance(component, SelectMenu):
+                    yield component
+
     async def delete(self, *, delay=None):
         """|coro|
 
@@ -1053,8 +1076,10 @@ class Message(Hashable):
         embed: Optional[:class:`Embed`]
             The new embed to replace the original with.
             Could be ``None`` to remove the embed.
-        components: List[:class:`discord.ActionRow`]
-            A list of :type:`discord.Actionrow`'s
+        embeds: Optional[List[:class:`Embed`]]
+            A list containing up to 10 embeds
+        components: List[Union[:class:`discord.ActionRow`, List]]
+            A list of :class:`discord.ActionRow`'s or a Lists with :class:`Button`'s or :class:`SelectMenu`.
         suppress: :class:`bool`
             Whether to suppress embeds for the message. This removes
             all the embeds if set to ``True``. If set to ``False``
@@ -1090,6 +1115,7 @@ class Message(Hashable):
         else:
             if content is not None:
                 fields['content'] = str(content)
+        raw_embeds = []
 
         try:
             embed = fields.pop('embed')
@@ -1097,15 +1123,18 @@ class Message(Hashable):
             pass
         else:
             if embed is not None:
-                fields['embeds'] = [embed.to_dict()]
+                raw_embeds.append(embed.to_dict())
 
         try:
-            embeds = fields['embeds']
+            embeds = fields.pop('embeds')
         except KeyError:
             pass
         else:
-            if embeds is not None and not embed:
-                fields['embeds'] = [e.to_dict() for e in embeds]
+            if embeds is not None:
+                raw_embeds.extend([e.to_dict() for e in embeds])
+        
+        if raw_embeds:
+            fields['embeds'] = raw_embeds
 
         try:
             components = fields['components']
@@ -1113,16 +1142,16 @@ class Message(Hashable):
             pass
         else:
             if components is not None:
-                from .components import Button, DropdownMenue, ActionRow
-                components_list = []
-                for component in ([components] if not type(components) == list else components):
-                    if isinstance(component, Button):
-                        components_list.append(component.to_dict())
-                    elif isinstance(component, DropdownMenue):
-                        components_list.append(component.to_dict())
+                _components = []
+                for component in (list(components) if not isinstance(components, list) else components):
+                    if isinstance(component, (Button, SelectMenu)):
+                        _components.extend(ActionRow(component).to_dict())
                     elif isinstance(component, ActionRow):
-                        components_list.extend(component.sendable())
-                fields['components'] = components_list
+                        _components.extend(component.to_dict())
+                    elif isinstance(component, list):
+                        _components.extend(ActionRow(*[obj for obj in component if isinstance(obj, (Button, SelectMenu))]).to_dict())
+                fields['components'] = _components
+
         try:
             suppress = fields.pop('suppress')
         except KeyError:
@@ -1146,28 +1175,35 @@ class Message(Hashable):
                     allowed_mentions = allowed_mentions.to_dict()
                 fields['allowed_mentions'] = allowed_mentions
 
-        is_interaction_responce = fields.pop('__is_interaction_responce', None)
-        if is_interaction_responce is True:
-            deffered = fields.pop('__deffered', False)
+        is_interaction_response = fields.pop('__is_interaction_response', None)
+        if is_interaction_response is True:
+            deferred = fields.pop('__deferred', False)
             use_webhook = fields.pop('__use_webhook', False)
             interaction_id = fields.pop('__interaction_id', None)
             interaction_token = fields.pop('__interaction_token', None)
             application_id = fields.pop('__application_id', None)
+            files = fields.pop('files', fields.pop('file', None))
+            if files and not isinstance(files, list):
+                files = [files]
             if fields:
                 try:
-                    data = await self._state.http.edit_interaction_response(use_webhook=use_webhook,
+                    payload = await self._state.http.edit_interaction_response(use_webhook=use_webhook,
                                                                             interaction_id=interaction_id,
                                                                             token=interaction_token,
                                                                             application_id=application_id,
-                                                                            deffered=deffered, **fields)
+                                                                            deferred=deferred, files=files, **fields)
                 except NotFound:
-                    is_interaction_responce = None
+                    is_interaction_response = None
                 else:
-                    [self.__setattr__(k, v) for k, v in fields.items()]
+                    if payload:
+                        self._update(payload)
+                    else:
+                        [self.__setattr__(k, v) for k, v in fields.items()]
 
-        elif is_interaction_responce is None:
+        if is_interaction_response is None:
             payload = await self._state.http.edit_message(self.channel.id, self.id, **fields)
             self._update(payload)
+
         if delete_after is not None:
             await self.delete(delay=delete_after)
 
@@ -1459,6 +1495,7 @@ def implement_partial_methods(cls):
 
 @implement_partial_methods
 class PartialMessage(Hashable):
+
     """Represents a partial message to aid with working messages when only
     a message and channel ID are present.
 
@@ -1562,6 +1599,28 @@ class PartialMessage(Hashable):
         data = await self._state.http.get_message(self.channel.id, self.id)
         return self._state.create_message(channel=self.channel, data=data)
 
+    @property
+    def all_components(self):
+        """Returns all :class:`Button`'s and :class:`SelectMenu`'s that are contained in the message"""
+        for action_row in self.components:
+            for component in action_row:
+                yield component
+
+    @property
+    def all_buttons(self):
+        """Returns all :class:`Button`'s that are contained in the message"""
+        for action_row in self.components:
+            for component in action_row:
+                yield component
+
+    @property
+    def all_select_menus(self):
+        """Returns all :class:`SelectMenu`'s that are contained in the message"""
+        for action_row in self.components:
+            for component in action_row:
+                if isinstance(component, SelectMenu):
+                    yield component
+
     async def edit(self, **fields):
         """|coro|
 
@@ -1580,8 +1639,10 @@ class PartialMessage(Hashable):
         embed: Optional[:class:`Embed`]
             The new embed to replace the original with.
             Could be ``None`` to remove the embed.
-        components: List[:class:`discord.ActionRow`]
-            A list of :type:`discord.Actionrow`'s
+        embeds: Optional[List[:class:`Embed`]]
+            A list containing up to 10 embeds
+        components: List[Union[:class:`discord.ActionRow`, List]]
+            A list of :class:`discord.ActionRow`'s or a Lists with :class:`Button`'s or :class:`SelectMenu`.
         suppress: :class:`bool`
             Whether to suppress embeds for the message. This removes
             all the embeds if set to ``True``. If set to ``False``
@@ -1623,36 +1684,41 @@ class PartialMessage(Hashable):
             if content is not None:
                 fields['content'] = str(content)
 
+        raw_embeds = []
+
         try:
             embed = fields.pop('embed')
         except KeyError:
             pass
         else:
             if embed is not None:
-                fields['embeds'] = [embed.to_dict()]
+                raw_embeds.append(embed.to_dict())
 
         try:
-            embeds = fields['embeds']
+            embeds = fields.pop('embeds')
         except KeyError:
             pass
         else:
-            if embeds is not None and not embed:
-                fields['embeds'] = [e.to_dict() for e in embeds]
-
+            if embeds is not None:
+                raw_embeds.extend([e.to_dict() for e in embeds])
+        
+        if raw_embeds:
+            fields['embeds'] = raw_embeds
+            
         try:
             components = fields['components']
         except KeyError:
             pass
         else:
-            _components = []
             if components is not None:
-                for component in ([components] if not type(components) == list else components):
-                    if isinstance(component, Button):
-                        _components.append(component.to_dict())
-                    elif isinstance(component, DropdownMenue):
-                        _components.append(component.to_dict())
+                _components = []
+                for component in (list(components) if not isinstance(components, list) else components):
+                    if isinstance(component, (Button, SelectMenu)):
+                        _components.extend(ActionRow(component).to_dict())
                     elif isinstance(component, ActionRow):
-                        _components.extend(component.sendable())
+                        _components.extend(component.to_dict())
+                    elif isinstance(component, list):
+                        _components.extend(ActionRow(*[obj for obj in component if isinstance(obj, (Button, SelectMenu))]).to_dict())
                 fields['components'] = _components
 
         try:
@@ -1678,31 +1744,36 @@ class PartialMessage(Hashable):
                     allowed_mentions = allowed_mentions.to_dict()
                 fields['allowed_mentions'] = allowed_mentions
 
-        is_interaction_responce = fields.pop('__is_interaction_responce', None)
-        if is_interaction_responce is True:
-            deffered = fields.pop('__deffered', False)
+        is_interaction_response = fields.pop('__is_interaction_response', None)
+        if is_interaction_response is True:
+            deferred = fields.pop('__deferred', False)
             use_webhook = fields.pop('__use_webhook', False)
             interaction_id = fields.pop('__interaction_id', None)
             interaction_token = fields.pop('__interaction_token', None)
             application_id = fields.pop('__application_id', None)
+            files = fields.pop('files', fields.pop('file', None))
+            if files and not isinstance(files, list):
+                files = [files]
             if fields:
                 try:
-                    data = await self._state.http.edit_interaction_response(use_webhook=use_webhook,
-                                                                            interaction_id=interaction_id,
-                                                                            token=interaction_token,
-                                                                            application_id=application_id,
-                                                                            deffered=deffered, **fields)
+                    payload = await self._state.http.edit_interaction_response(use_webhook=use_webhook,
+                                                                               interaction_id=interaction_id,
+                                                                               token=interaction_token,
+                                                                               application_id=application_id,
+                                                                               deferred=deferred, files=files, **fields)
                 except NotFound:
-                    is_interaction_responce = None
+                    is_interaction_response = None
                 else:
-                    [self.__setattr__(k, v) for k, v in fields.items()]
+                    if payload:
+                        self._update(payload)
+                    else:
+                        [self.__setattr__(k, v) for k, v in fields.items()]
 
-        elif is_interaction_responce is None:
+        if is_interaction_response is None:
             payload = await self._state.http.edit_message(self.channel.id, self.id, **fields)
             self._update(payload)
 
         if delete_after is not None:
             await self.delete(delay=delete_after)
 
-        if fields:
-            return self._state.create_message(channel=self.channel, data=data)
+        return self._state.create_message(channel=self.channel, data=payload)
